"use strict";

exports.__esModule = true;
exports.default = createContext;

var _component = _interopRequireDefault(require("./vdom/component"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _inheritsLoose(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; subClass.__proto__ = superClass; }

var ValueEmitter =
/*#__PURE__*/
function () {
  function ValueEmitter(defaultValue) {
    this.handlers = [];
    this.value = defaultValue;
  }

  var _proto = ValueEmitter.prototype;

  _proto.on = function on(handler) {
    this.handlers.push(handler);
  };

  _proto.off = function off(handler) {
    this.handlers = this.handlers.filter(function (h) {
      return h !== handler;
    });
  };

  _proto.emit = function emit() {
    var _this = this;

    this.handlers.forEach(function (handler) {
      return handler(_this.value);
    });
  };

  return ValueEmitter;
}();

var uniqueId = 0;

function createContext(defaultValue) {
  var _Provider$childContex, _Consumer$contextType;

  var contextProp = '__context_' + uniqueId++ + '__';

  var Provider =
  /*#__PURE__*/
  function (_Component) {
    _inheritsLoose(Provider, _Component);

    function Provider() {
      var _this2;

      _this2 = _Component.call(this) || this;
      _this2.emitter = new ValueEmitter(defaultValue);
      return _this2;
    }

    var _proto2 = Provider.prototype;

    _proto2.getChildContext = function getChildContext() {
      var _ref;

      return _ref = {}, _ref[contextProp] = this.emitter, _ref;
    };

    _proto2.componentWillMount = function componentWillMount() {
      if (this.props.value !== undefined) {
        this.emitter.value = this.props.value;
      }
    };

    _proto2.componentWillReceiveProps = function componentWillReceiveProps(nextProps) {
      if (this.props.value !== nextProps.value) {
        this.emitter.value = nextProps.value;
      }
    };

    _proto2.componentDidUpdate = function componentDidUpdate(prevProps) {
      if (this.props.value !== prevProps.value) {
        this.emitter.emit();
      }
    };

    _proto2.render = function render() {
      return this.props.children;
    };

    return Provider;
  }(_component.default);

  Provider.childContextTypes = (_Provider$childContex = {}, _Provider$childContex[contextProp] = function () {}, _Provider$childContex);
  Provider.contextProp = contextProp;
  Provider.defaultValue = defaultValue;

  var Consumer =
  /*#__PURE__*/
  function (_Component2) {
    _inheritsLoose(Consumer, _Component2);

    function Consumer(props, context) {
      var _this3;

      _this3 = _Component2.call(this, props, context) || this;
      _this3.state = {
        value: _this3.readContext(_this3.context)
      };

      _this3.onUpdate = function (value) {
        return _this3.state.value !== value && _this3.setState({
          value: value
        });
      };

      return _this3;
    }

    var _proto3 = Consumer.prototype;

    _proto3.readContext = function readContext(context) {
      return context[contextProp] ? context[contextProp].value : defaultValue;
    };

    _proto3.componentDidMount = function componentDidMount() {
      if (this.context[contextProp]) {
        this.context[contextProp].on(this.onUpdate);
      }
    };

    _proto3.componentWillReceiveProps = function componentWillReceiveProps(nextProps, nextContext) {
      var newContextValue = this.readContext(nextContext);

      if (this.state.value !== newContextValue) {
        this.setState({
          value: newContextValue
        });
      }
    };

    _proto3.componentWillUnmount = function componentWillUnmount() {
      if (this.context[contextProp]) {
        this.context[contextProp].off(this.onUpdate);
      }
    };

    _proto3.render = function render() {
      var children = this.props.children;
      var consumer = Array.isArray(children) ? children[0] : children;

      if (typeof consumer === 'function') {
        return consumer(this.state.value);
      }
    };

    return Consumer;
  }(_component.default);

  Consumer.contextTypes = (_Consumer$contextType = {}, _Consumer$contextType[contextProp] = function () {}, _Consumer$contextType);
  return {
    Provider: Provider,
    Consumer: Consumer
  };
}