"use strict";

exports.__esModule = true;
exports.default = createElement;

var _host = _interopRequireDefault(require("./vdom/host"));

var _element = _interopRequireDefault(require("./vdom/element"));

var _flattenChildren = _interopRequireDefault(require("./vdom/flattenChildren"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var RESERVED_PROPS = {
  key: true,
  ref: true
};

function getRenderErrorInfo() {
  var ownerComponent = _host.default.owner;

  if (ownerComponent) {
    var name = ownerComponent.getName();

    if (name) {
      return ' Check the render method of `' + name + '`.';
    }
  }

  return '';
}

function createElement(type, config, children) {
  if (type == null) {
    throw Error('createElement: type should not be null or undefined.' + getRenderErrorInfo());
  } // Reserved names are extracted


  var props = {};
  var propName;
  var key = null;
  var ref = null;
  var ownerComponent = _host.default.owner;

  if (config != null) {
    ref = config.ref === undefined ? null : config.ref;
    key = config.key === undefined ? null : String(config.key);

    if (typeof ref === 'string' && !ownerComponent) {
      console.warn('createElement: adding a string ref outside the component’s render() method.');
    } // Remaining properties are added to a new props object


    for (propName in config) {
      if (!RESERVED_PROPS[propName]) {
        props[propName] = config[propName];
      }
    }
  } // Children arguments can be more than one


  var childrenLength = arguments.length - 2;

  if (childrenLength > 0) {
    if (childrenLength === 1 && !Array.isArray(children)) {
      props.children = children;
    } else {
      var childArray = children;

      if (childrenLength > 1) {
        childArray = new Array(childrenLength);

        for (var i = 0; i < childrenLength; i++) {
          childArray[i] = arguments[i + 2];
        }
      }

      props.children = (0, _flattenChildren.default)(childArray);
    }
  } // Resolve default props


  if (type && type.defaultProps) {
    var defaultProps = type.defaultProps;

    for (propName in defaultProps) {
      if (props[propName] === undefined) {
        props[propName] = defaultProps[propName];
      }
    }
  }

  return new _element.default(type, key, ref, props, ownerComponent);
}