"use strict";

exports.__esModule = true;
exports.default = void 0;

var _host = _interopRequireDefault(require("./host"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Base Component
 */
var BaseComponent =
/*#__PURE__*/
function () {
  function BaseComponent(element) {
    this._currentElement = element;
  }

  var _proto = BaseComponent.prototype;

  _proto.initComponent = function initComponent(parent, parentInstance, context) {
    this._parent = parent;
    this._parentInstance = parentInstance;
    this._context = context;
    this._mountID = _host.default.mountID++;
  };

  _proto.destoryComponent = function destoryComponent() {
    if (process.env.NODE_ENV !== 'production') {
      _host.default.hook.Reconciler.unmountComponent(this);
    }

    this._currentElement = null;
    this._nativeNode = null;
    this._parent = null;
    this._parentInstance = null;
    this._context = null;

    if (this._instance) {
      this._instance._internal = null;
      this._instance = null;
    }
  };

  _proto.mountComponent = function mountComponent(parent, parentInstance, context, nativeNodeMounter) {
    this.initComponent(parent, parentInstance, context);
    this.mountNativeNode(nativeNodeMounter);

    if (process.env.NODE_ENV !== 'production') {
      _host.default.hook.Reconciler.mountComponent(this);
    }

    var instance = {
      _internal: this
    };
    return instance;
  };

  _proto.updateComponent = function updateComponent() {// Noop
  };

  _proto.unmountComponent = function unmountComponent(shouldNotRemoveChild) {
    if (this._nativeNode && !shouldNotRemoveChild) {
      _host.default.driver.removeChild(this._nativeNode, this._parent);
    }

    this.destoryComponent();
  };

  _proto.getName = function getName() {
    var currentElement = this._currentElement;
    var instance = this._instance;
    var type = currentElement && currentElement.type;
    var constructor = instance && instance.constructor;
    return type && type.displayName || constructor && constructor.displayName || type && type.name || constructor && constructor.name || type || // Native component's name is type
    currentElement;
  };

  _proto.mountNativeNode = function mountNativeNode(nativeNodeMounter) {
    var nativeNode = this.getNativeNode();
    var parent = this._parent;

    if (nativeNodeMounter) {
      nativeNodeMounter(nativeNode, parent);
    } else {
      _host.default.driver.appendChild(nativeNode, parent);
    }
  };

  _proto.getNativeNode = function getNativeNode() {
    if (this._nativeNode == null) {
      this._nativeNode = this.createNativeNode();
    }

    return this._nativeNode;
  };

  _proto.getPublicInstance = function getPublicInstance() {
    return this.getNativeNode();
  };

  return BaseComponent;
}();

var _default = BaseComponent;
exports.default = _default;