"use strict";

exports.__esModule = true;
exports.default = void 0;

var _reactive = _interopRequireDefault(require("./reactive"));

var _updater = _interopRequireDefault(require("./updater"));

var _host = _interopRequireDefault(require("./host"));

var _ref = _interopRequireDefault(require("./ref"));

var _instantiateComponent = _interopRequireWildcard(require("./instantiateComponent"));

var _shouldUpdateComponent = _interopRequireDefault(require("./shouldUpdateComponent"));

var _shallowEqual = _interopRequireDefault(require("./shallowEqual"));

var _base = _interopRequireDefault(require("./base"));

var _toArray = _interopRequireDefault(require("./toArray"));

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _inheritsLoose(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; subClass.__proto__ = superClass; }

function performInSandbox(fn, instance, callback) {
  try {
    return fn();
  } catch (e) {
    if (callback) {
      callback(e);
    } else {
      handleError(instance, e);
    }
  }
}

function handleError(instance, error) {
  var boundary;

  while (instance) {
    var internal = instance._internal;

    if (typeof instance.componentDidCatch === 'function') {
      boundary = instance;
      break;
    } else if (internal && internal._parentInstance) {
      instance = internal._parentInstance;
    } else {
      break;
    }
  }

  if (boundary) {
    // Should not attempt to recover an unmounting error boundary
    var boundaryInternal = boundary._internal;

    if (boundaryInternal) {
      var callbackQueue = boundaryInternal._pendingCallbacks || (boundaryInternal._pendingCallbacks = []);
      callbackQueue.push(function () {
        return boundary.componentDidCatch(error);
      });
    }
  } else {
    // Do not break when error happens
    setTimeout(function () {
      throw error;
    }, 0);
  }
}

var measureLifeCycle;

if (process.env.NODE_ENV !== 'production') {
  measureLifeCycle = function measureLifeCycle(callback, instanceID, type) {
    _host.default.measurer && _host.default.measurer.beforeLifeCycle(instanceID, type);
    callback();
    _host.default.measurer && _host.default.measurer.afterLifeCycle(instanceID, type);
  };
}
/**
 * Composite Component
 */


var CompositeComponent =
/*#__PURE__*/
function (_BaseComponent) {
  _inheritsLoose(CompositeComponent, _BaseComponent);

  function CompositeComponent() {
    return _BaseComponent.apply(this, arguments) || this;
  }

  var _proto = CompositeComponent.prototype;

  _proto.mountComponent = function mountComponent(parent, parentInstance, context, nativeNodeMounter) {
    var _this = this;

    this.initComponent(parent, parentInstance, context);
    this._updateCount = 0;

    if (process.env.NODE_ENV !== 'production') {
      _host.default.measurer && _host.default.measurer.beforeMountComponent(this._mountID, this);
    }

    var currentElement = this._currentElement;
    var Component = currentElement.type;
    var ref = currentElement.ref;
    var publicProps = currentElement.props;
    var componentPrototype = Component.prototype; // Context process

    var publicContext = this._processContext(context); // Initialize the public class


    var instance;
    var renderedElement;

    try {
      if (componentPrototype && componentPrototype.render) {
        // Class Component instance
        instance = new Component(publicProps, publicContext);
      } else if (typeof Component === 'function') {
        // Functional reactive component with hooks
        instance = new _reactive.default(Component, ref);
      } else {
        (0, _instantiateComponent.throwInvalidComponentError)(Component);
      }
    } catch (e) {
      return handleError(parentInstance, e);
    } // These should be set up in the constructor, but as a convenience for
    // simpler class abstractions, we set them up after the fact.


    instance.props = publicProps;
    instance.context = publicContext;
    instance.refs = {}; // Inject the updater into instance

    instance.updater = _updater.default;
    instance._internal = this;
    this._instance = instance; // Init state, must be set to an object or null

    var initialState = instance.state;

    if (initialState === undefined) {
      // TODO clone the state?
      instance.state = initialState = null;
    }

    var error = null;

    var errorCallback = function errorCallback(e) {
      error = e;
    };

    if (instance.componentWillMount) {
      performInSandbox(function () {
        if (process.env.NODE_ENV !== 'production') {
          measureLifeCycle(function () {
            instance.componentWillMount();
          }, _this._mountID, 'componentWillMount');
        } else {
          instance.componentWillMount();
        }
      }, instance, errorCallback);
    }

    if (renderedElement == null) {
      _host.default.owner = this; // Process pending state when call setState in componentWillMount

      instance.state = this._processPendingState(publicProps, publicContext);
      performInSandbox(function () {
        if (process.env.NODE_ENV !== 'production') {
          measureLifeCycle(function () {
            renderedElement = instance.render();
          }, _this._mountID, 'render');
        } else {
          renderedElement = instance.render();
        }
      }, instance, errorCallback);
      _host.default.owner = null;
    }

    this._renderedComponent = (0, _instantiateComponent.default)(renderedElement);

    this._renderedComponent.mountComponent(this._parent, instance, this._processChildContext(context), nativeNodeMounter);

    if (error) {
      handleError(instance, error);
    }

    if (!currentElement.type.forwardRef && ref) {
      _ref.default.attach(currentElement._owner, ref, this);
    }

    if (instance.componentDidMount) {
      performInSandbox(function () {
        if (process.env.NODE_ENV !== 'production') {
          measureLifeCycle(function () {
            instance.componentDidMount();
          }, _this._mountID, 'componentDidMount');
        } else {
          instance.componentDidMount();
        }
      }, instance);
    } // Trigger setState callback in componentWillMount or boundary callback after rendered


    var callbacks = this._pendingCallbacks;

    if (callbacks) {
      this._pendingCallbacks = null;

      _updater.default.runCallbacks(callbacks, instance);
    }

    if (process.env.NODE_ENV !== 'production') {
      _host.default.hook.Reconciler.mountComponent(this);

      _host.default.measurer && _host.default.measurer.afterMountComponent(this._mountID);
    }

    return instance;
  };

  _proto.unmountComponent = function unmountComponent(shouldNotRemoveChild) {
    var instance = this._instance; // Unmounting a composite component maybe not complete mounted
    // when throw error in component constructor stage

    if (instance && instance.componentWillUnmount) {
      performInSandbox(function () {
        instance.componentWillUnmount();
      }, instance);
    }

    if (this._renderedComponent != null) {
      var currentElement = this._currentElement;
      var ref = currentElement.ref;

      if (!currentElement.type.forwardRef && ref) {
        _ref.default.detach(currentElement._owner, ref, this);
      }

      this._renderedComponent.unmountComponent(shouldNotRemoveChild);

      this._renderedComponent = null;
    } // Reset pending fields
    // Even if this component is scheduled for another async update,
    // it would still be ignored because these fields are reset.


    this._pendingStateQueue = null;
    this._isPendingForceUpdate = false;
    this.destoryComponent();
  }
  /**
   * Filters the context object to only contain keys specified in
   * `contextTypes`
   */
  ;

  _proto._processContext = function _processContext(context) {
    var Component = this._currentElement.type;
    var contextTypes = Component.contextTypes;

    if (!contextTypes) {
      return {};
    }

    var maskedContext = {};

    for (var contextName in contextTypes) {
      maskedContext[contextName] = context[contextName];
    }

    return maskedContext;
  };

  _proto._processChildContext = function _processChildContext(currentContext) {
    var instance = this._instance; // The getChildContext method context should be current instance

    var childContext = instance.getChildContext && instance.getChildContext();

    if (childContext) {
      return Object.assign({}, currentContext, childContext);
    }

    return currentContext;
  };

  _proto._processPendingState = function _processPendingState(props, context) {
    var instance = this._instance;
    var queue = this._pendingStateQueue;

    if (!queue) {
      return instance.state;
    } // Reset pending queue


    this._pendingStateQueue = null;
    var nextState = Object.assign({}, instance.state);

    for (var i = 0; i < queue.length; i++) {
      var partial = queue[i];
      Object.assign(nextState, typeof partial === 'function' ? partial.call(instance, nextState, props, context) : partial);
    }

    return nextState;
  };

  _proto.updateComponent = function updateComponent(prevElement, nextElement, prevUnmaskedContext, nextUnmaskedContext) {
    var instance = this._instance; // Maybe update component that has already been unmounted or failed mount.

    if (!instance) {
      return;
    }

    if (process.env.NODE_ENV !== 'production') {
      _host.default.measurer && _host.default.measurer.beforeUpdateComponent(this._mountID, this);
    }

    var willReceive = false;
    var nextContext;
    var nextProps; // Determine if the context has changed or not

    if (this._context === nextUnmaskedContext) {
      nextContext = instance.context;
    } else {
      nextContext = this._processContext(nextUnmaskedContext);
      willReceive = true;
    } // Distinguish between a props update versus a simple state update


    if (prevElement === nextElement) {
      // Skip checking prop types again -- we don't read component.props to avoid
      // warning for DOM component props in this upgrade
      nextProps = nextElement.props;
    } else {
      nextProps = nextElement.props;
      willReceive = true;
    }

    var hasReceived = willReceive && instance.componentWillReceiveProps;

    if (hasReceived) {
      // Calling this.setState() within componentWillReceiveProps will not trigger an additional render.
      this._isPendingState = true;
      performInSandbox(function () {
        instance.componentWillReceiveProps(nextProps, nextContext);
      }, instance);
      this._isPendingState = false;
    } // Update refs


    if (this._currentElement.type.forwardRef) {
      instance.prevForwardRef = prevElement.ref;
      instance.forwardRef = nextElement.ref;
    } else {
      _ref.default.update(prevElement, nextElement, this);
    } // Shoud update default


    var shouldUpdate = true;
    var prevProps = instance.props;
    var prevState = instance.state; // TODO: could delay execution processPendingState

    var nextState = this._processPendingState(nextProps, nextContext); // ShouldComponentUpdate is not called when forceUpdate is used


    if (!this._isPendingForceUpdate) {
      if (instance.shouldComponentUpdate) {
        shouldUpdate = performInSandbox(function () {
          return instance.shouldComponentUpdate(nextProps, nextState, nextContext);
        }, instance);
      } else if (instance.isPureComponent) {
        // Pure Component
        shouldUpdate = !(0, _shallowEqual.default)(prevProps, nextProps) || !(0, _shallowEqual.default)(prevState, nextState);
      }
    }

    if (shouldUpdate) {
      this._isPendingForceUpdate = false; // Will set `this.props`, `this.state` and `this.context`.

      var prevContext = instance.context; // Cannot use this.setState() in componentWillUpdate.
      // If need to update state in response to a prop change, use componentWillReceiveProps instead.

      performInSandbox(function () {
        if (instance.componentWillUpdate) {
          instance.componentWillUpdate(nextProps, nextState, nextContext);
        }
      }, instance); // Replace with next

      this._currentElement = nextElement;
      this._context = nextUnmaskedContext;
      instance.props = nextProps;
      instance.state = nextState;
      instance.context = nextContext;

      this._updateRenderedComponent(nextUnmaskedContext);

      performInSandbox(function () {
        if (instance.componentDidUpdate) {
          instance.componentDidUpdate(prevProps, prevState, prevContext);
        }
      }, instance);
      this._updateCount++;
    } else {
      // If it's determined that a component should not update, we still want
      // to set props and state but we shortcut the rest of the update.
      this._currentElement = nextElement;
      this._context = nextUnmaskedContext;
      instance.props = nextProps;
      instance.state = nextState;
      instance.context = nextContext;
    } // Flush setState callbacks set in componentWillReceiveProps or boundary callback


    var callbacks = this._pendingCallbacks;

    if (callbacks) {
      this._pendingCallbacks = null;

      _updater.default.runCallbacks(callbacks, instance);
    }

    if (process.env.NODE_ENV !== 'production') {
      _host.default.measurer && _host.default.measurer.afterUpdateComponent(this._mountID);

      _host.default.hook.Reconciler.receiveComponent(this);
    }
  }
  /**
   * Call the component's `render` method and update the DOM accordingly.
   */
  ;

  _proto._updateRenderedComponent = function _updateRenderedComponent(context) {
    var _this2 = this;

    var prevRenderedComponent = this._renderedComponent;
    var prevRenderedElement = prevRenderedComponent._currentElement;
    var instance = this._instance;
    var nextRenderedElement;
    _host.default.owner = this;
    performInSandbox(function () {
      if (process.env.NODE_ENV !== 'production') {
        measureLifeCycle(function () {
          nextRenderedElement = instance.render();
        }, _this2._mountID, 'render');
      } else {
        nextRenderedElement = instance.render();
      }
    }, instance);
    _host.default.owner = null;

    if ((0, _shouldUpdateComponent.default)(prevRenderedElement, nextRenderedElement)) {
      var prevRenderedUnmaskedContext = prevRenderedComponent._context;

      var nextRenderedUnmaskedContext = this._processChildContext(context);

      if (prevRenderedElement !== nextRenderedElement || prevRenderedUnmaskedContext !== nextRenderedUnmaskedContext) {
        prevRenderedComponent.updateComponent(prevRenderedElement, nextRenderedElement, prevRenderedUnmaskedContext, nextRenderedUnmaskedContext);
      }

      if (process.env.NODE_ENV !== 'production') {
        _host.default.measurer && _host.default.measurer.recordOperation({
          instanceID: this._mountID,
          type: 'update component',
          payload: {}
        });
      }
    } else {
      var prevNativeNode = prevRenderedComponent.getNativeNode();
      prevRenderedComponent.unmountComponent(true);
      this._renderedComponent = (0, _instantiateComponent.default)(nextRenderedElement);

      this._renderedComponent.mountComponent(this._parent, instance, this._processChildContext(context), function (newNativeNode, parent) {
        prevNativeNode = (0, _toArray.default)(prevNativeNode);
        newNativeNode = (0, _toArray.default)(newNativeNode);
        var driver = _host.default.driver; // If the new length large then prev

        var lastNativeNode;

        for (var i = 0; i < newNativeNode.length; i++) {
          var nativeNode = newNativeNode[i];

          if (prevNativeNode[i]) {
            driver.replaceChild(nativeNode, prevNativeNode[i]);
          } else if (lastNativeNode) {
            driver.insertAfter(nativeNode, lastNativeNode);
          } else {
            driver.appendChild(nativeNode, parent);
          }

          lastNativeNode = nativeNode;
        } // If the new length less then prev


        if (newNativeNode.length < prevNativeNode.length) {
          for (var _i = newNativeNode.length; _i < prevNativeNode.length; _i++) {
            driver.removeChild(prevNativeNode[_i]);
          }
        }
      });
    }
  };

  _proto.getNativeNode = function getNativeNode() {
    // 123
    var renderedComponent = this._renderedComponent;

    if (renderedComponent) {
      return renderedComponent.getNativeNode();
    }
  };

  _proto.getPublicInstance = function getPublicInstance() {
    var instance = this._instance; // The functional components cannot be given refs

    if (instance instanceof _reactive.default) {
      return null;
    }

    return instance;
  };

  return CompositeComponent;
}(_base.default);

var _default = CompositeComponent;
exports.default = _default;