"use strict";

exports.__esModule = true;
exports.default = void 0;

var _host = _interopRequireDefault(require("./host"));

var _createElement = _interopRequireDefault(require("../createElement"));

var _instantiateComponent = _interopRequireDefault(require("./instantiateComponent"));

var _root = _interopRequireDefault(require("./root"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Instance manager
 */
var KEY = '__r';
var _default = {
  set: function set(node, instance) {
    if (!node[KEY]) {
      node[KEY] = instance; // Record root instance to roots map

      if (instance.rootID) {
        _host.default.rootInstances[instance.rootID] = instance;
        _host.default.rootComponents[instance.rootID] = instance._internal;
      }
    }
  },
  get: function get(node) {
    return node[KEY];
  },
  remove: function remove(node) {
    var instance = this.get(node);

    if (instance) {
      node[KEY] = null;

      if (instance.rootID) {
        delete _host.default.rootComponents[instance.rootID];
        delete _host.default.rootInstances[instance.rootID];
      }
    }
  },
  mount: function mount(element, container, parentInstance) {
    if (process.env.NODE_ENV !== 'production') {
      _host.default.measurer && _host.default.measurer.beforeRender();
    }

    var driver = _host.default.driver; // Before render callback

    driver.beforeRender && driver.beforeRender(); // Real native root node is body

    if (container == null) {
      container = driver.createBody();
    } // Get the context from the conceptual parent component.


    var parentContext;

    if (parentInstance) {
      var parentInternal = parentInstance._internal;
      parentContext = parentInternal._processChildContext(parentInternal._context);
    } // Update root component


    var prevRootInstance = this.get(container);

    if (prevRootInstance && prevRootInstance.rootID) {
      if (parentContext) {
        // Using _penddingContext to pass new context
        prevRootInstance._internal._penddingContext = parentContext;
      }

      prevRootInstance.update(element);
      return prevRootInstance;
    } // Init root component with empty children


    var renderedComponent = (0, _instantiateComponent.default)((0, _createElement.default)(_root.default));
    var defaultContext = parentContext || {};
    var rootInstance = renderedComponent.mountComponent(container, null, defaultContext);
    this.set(container, rootInstance); // Mount new element through update queue avoid when there is in rendering phase

    rootInstance.update(element); // After render callback

    driver.afterRender && driver.afterRender(rootInstance);

    if (process.env.NODE_ENV !== 'production') {
      // Devtool render new root hook
      _host.default.hook.Mount._renderNewRootComponent(rootInstance._internal);

      _host.default.measurer && _host.default.measurer.afterRender();
    }

    return rootInstance;
  }
};
exports.default = _default;