"use strict";

exports.__esModule = true;
exports.default = void 0;

var _host = _interopRequireDefault(require("./host"));

var _component = _interopRequireDefault(require("./component"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _inheritsLoose(subClass, superClass) { subClass.prototype = Object.create(superClass.prototype); subClass.prototype.constructor = subClass; subClass.__proto__ = superClass; }

var RE_RENDER_LIMIT = 24;
/**
 * Functional Reactive Component Class Wrapper
 */

var ReactiveComponent =
/*#__PURE__*/
function (_Component) {
  _inheritsLoose(ReactiveComponent, _Component);

  function ReactiveComponent(pureRender, ref) {
    var _this;

    _this = _Component.call(this) || this; // A pure function

    _this._render = pureRender;
    _this._hookID = 0; // Number of rerenders

    _this._reRenders = 0;
    _this._hooks = {}; // Handles store

    _this.didMount = [];
    _this.didUpdate = [];
    _this.willUnmount = []; // Is render scheduled

    _this.isScheduled = false;

    if (pureRender.forwardRef) {
      _this.prevForwardRef = _this.forwardRef = ref;
    }

    var compares = pureRender.compares;

    if (compares) {
      _this.shouldComponentUpdate = function (nextProps) {
        // Process composed compare
        var arePropsEqual = true; // Compare push in and pop out

        for (var i = compares.length - 1; i > -1; i--) {
          if (arePropsEqual = compares[i](_this.props, nextProps)) {
            break;
          }

          ;
        }

        return !arePropsEqual || _this.prevForwardRef !== _this.forwardRef;
      };
    }

    return _this;
  }

  var _proto = ReactiveComponent.prototype;

  _proto.getHooks = function getHooks() {
    return this._hooks;
  };

  _proto.getHookID = function getHookID() {
    return ++this._hookID;
  };

  _proto.readContext = function readContext(context) {
    var _this2 = this;

    var Provider = context.Provider;
    var unmaskContext = this._internal._context;
    var contextProp = Provider.contextProp;
    var contextEmitter = unmaskContext[contextProp];

    if (contextEmitter) {
      var mountId = this._internal._mountID;

      if (!contextEmitter[mountId]) {
        // One context one updater bind
        contextEmitter[mountId] = {};

        var contextUpdater = function contextUpdater(newContext) {
          if (newContext !== contextEmitter[mountId].renderedContext) {
            _this2.update();
          }
        };

        contextEmitter.on(contextUpdater);
        this.willUnmount.push(function () {
          delete contextEmitter[mountId];
          contextEmitter.off(contextUpdater);
        });
      }

      return contextEmitter[mountId].renderedContext = contextEmitter.value;
    }

    return Provider.defaultValue;
  };

  _proto.componentDidMount = function componentDidMount() {
    this.didMount.forEach(function (handler) {
      return handler();
    });
  };

  _proto.componentDidUpdate = function componentDidUpdate() {
    this.didUpdate.forEach(function (handler) {
      return handler();
    });
  };

  _proto.componentWillUnmount = function componentWillUnmount() {
    this.willUnmount.forEach(function (handler) {
      return handler();
    });
  };

  _proto.update = function update() {
    this.forceUpdate();
  };

  _proto.render = function render() {
    if (process.env.NODE_ENV !== 'production') {
      _host.default.measurer && _host.default.measurer.beforeRender();
    }

    this._hookID = 0;
    this._reRenders = 0;
    this.isScheduled = false;

    var children = this._render(this.props, this.forwardRef ? this.forwardRef : this.context);

    while (this.isScheduled) {
      this._reRenders++;

      if (this._reRenders > RE_RENDER_LIMIT) {
        throw Error('Too many re-renders, the number of renders is limited to prevent an infinite loop.');
      }

      this._hookID = 0;
      this.isScheduled = false;
      children = this._render(this.props, this.forwardRef ? this.forwardRef : this.context);
    }

    return children;
  };

  return ReactiveComponent;
}(_component.default);

var _default = ReactiveComponent;
exports.default = _default;